package de.ddd.android;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import android.graphics.ImageFormat;
import android.hardware.Camera.Parameters;
import android.util.Log;
import crimsonwoods.android.libs.uvccap.PixelFormat;
import crimsonwoods.android.libs.uvccap.UVCCamera;

public class Camera {
	private UVCCamera uvcCamera;
	private boolean isPreviewing = false;
	private PreviewThread previewThread;
	private PreviewCallback previewCallback;
	private AutoFocusCallback autoFocusCallback;

	public Camera(int id, boolean isUvc) {
		uvcCamera = UVCCamera.open(id);
	}

	@Override
	protected void finalize() throws Throwable {
		try {
			super.finalize();
		} finally {
			release();
		}
	}

	public int get_frame_size()
	{
		return this.uvcCamera.getFrameSize();
	}
	
	public synchronized void init(int width, int height) throws IOException {
		if (null != uvcCamera) {
			uvcCamera.init(width, height, PixelFormat.MJPG);
			if (uvcCamera.getPixelFormat() != PixelFormat.MJPG) {
				uvcCamera.release();
				throw new IOException("Unsupported pixel format.");
			}
		}
	}

	public synchronized void release() {
		if (null != uvcCamera) {
			uvcCamera.release();
			uvcCamera = null;
		}
	}

	public synchronized void startPreview() {
		if (isPreviewing) {
			return;
		}
		if (null != uvcCamera) {
			if (null == previewThread) {
				previewThread = new PreviewThread(uvcCamera);
			}
			if (null != previewCallback) {
				previewThread.setPreviewCallback(previewCallback);
			}
			previewThread.start();
			isPreviewing = true;
		}
	}

	public void stopPreview() {
		synchronized (this) {
			if (!isPreviewing) {
				return;
			}
			if (null == uvcCamera) {
				return;
			}
			previewThread.interrupt();
		}

		if (Thread.currentThread().getId() != previewThread.getId()) {
			try {
				previewThread.join();
			} catch (InterruptedException e) {
			} finally {
				previewThread = null;
			}
		}
	}

	public synchronized void setPreviewCallback(PreviewCallback cb) {
		previewCallback = cb;
		if (null != uvcCamera) {
			if (null == previewThread) {
				previewThread = new PreviewThread(uvcCamera);
			}
			previewThread.setPreviewCallback(cb);
		}
	}

	public synchronized void autoFocus(AutoFocusCallback cb) {

	}

	public synchronized void cancelAutoFocus() {

	}

	public synchronized int getPreviewWidth() {
		if (null != uvcCamera) {
			return uvcCamera.getWidth();
		}
		throw new IllegalStateException();
	}

	public synchronized int getPreviewHeight() {
		if (null != uvcCamera) {
			return uvcCamera.getHeight();
		}
		throw new IllegalStateException();
	}

	public synchronized PixelFormat getPixelFormat() {
		if (null != uvcCamera) {
			return uvcCamera.getPixelFormat();
		}
		return PixelFormat.UNKNOWN;
	}

	private final class PreviewThread extends Thread {
		private UVCCamera camera_;
		private PreviewCallback callback_;

		PreviewThread(UVCCamera camera) {
			camera_ = camera;
			setName(getClass().getSimpleName());
		}

		public synchronized void setPreviewCallback(PreviewCallback cb) {
			callback_ = cb;
		}

		private void callback(byte[] pixels, int framesize) {
			final PreviewCallback cb;
			synchronized (this) {
				if (null == callback_) {
					return;
				}
				cb = callback_;
			}
			cb.onPreviewFrame(pixels, Camera.this, framesize);
		}

		@Override
		public void run() {
			final byte[][] pixels = new byte[4][camera_.getFrameSize()];
			int idx = 0;
			try {
				while (!Thread.interrupted()) {
					int framesize = camera_.capture(pixels[idx]);
					//Log.i("dddapp", "framesize:" + framesize);
					callback(pixels[idx], framesize);
					idx = (idx + 1) % pixels.length;
				}
				camera_.release();
				//Log.i("Camera", "" + camera_.getFrameSize());
			} catch (Throwable t) {
				//Log.e(t, "Uncaught exception.");
			}
		}
	}

	public interface PreviewCallback {
		void onPreviewFrame(byte[] data, Camera camera, int framesize);
	}

	public interface AutoFocusCallback {
		void onAutoFocus(boolean success, Camera camera);
	}

}